# This file was auto-generated by Fern from our API Definition.

import typing

import httpx
from ..environment import CandidApiClientEnvironment
from .http_client import AsyncHttpClient, HttpClient


class BaseClientWrapper:
    def __init__(
        self,
        *,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        environment: CandidApiClientEnvironment,
        timeout: typing.Optional[float] = None,
    ):
        self._token = token
        self._environment = environment
        self._timeout = timeout

    def get_headers(self) -> typing.Dict[str, str]:
        headers: typing.Dict[str, str] = {
            "User-Agent": "candidhealth/1.14.4",
            "X-Fern-Language": "Python",
            "X-Fern-SDK-Name": "candidhealth",
            "X-Fern-SDK-Version": "1.14.4",
        }
        token = self._get_token()
        if token is not None:
            headers["Authorization"] = f"Bearer {token}"
        return headers

    def _get_token(self) -> typing.Optional[str]:
        if isinstance(self._token, str) or self._token is None:
            return self._token
        else:
            return self._token()

    def get_environment(self) -> CandidApiClientEnvironment:
        return self._environment

    def get_timeout(self) -> typing.Optional[float]:
        return self._timeout


class SyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        environment: CandidApiClientEnvironment,
        timeout: typing.Optional[float] = None,
        httpx_client: httpx.Client,
    ):
        super().__init__(token=token, environment=environment, timeout=timeout)
        self.httpx_client = HttpClient(
            httpx_client=httpx_client, base_headers=self.get_headers, base_timeout=self.get_timeout
        )


class AsyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        environment: CandidApiClientEnvironment,
        timeout: typing.Optional[float] = None,
        httpx_client: httpx.AsyncClient,
    ):
        super().__init__(token=token, environment=environment, timeout=timeout)
        self.httpx_client = AsyncHttpClient(
            httpx_client=httpx_client, base_headers=self.get_headers, base_timeout=self.get_timeout
        )



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ReferralType(str, enum.Enum):
    DIRECTED = "DIRECTED"
    ROTATION = "ROTATION"
    OVERNIGHT = "OVERNIGHT"
    _UNKNOWN = "__REFERRALTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ReferralType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        directed: typing.Callable[[], T_Result],
        rotation: typing.Callable[[], T_Result],
        overnight: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ReferralType.DIRECTED:
            return directed()
        if self is ReferralType.ROTATION:
            return rotation()
        if self is ReferralType.OVERNIGHT:
            return overnight()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.coverage_id import CoverageId


class FilingOrder(UniversalBaseModel):
    """
    The patient's active coverages, in order of primary, secondary, etc.
    """

    coverages: typing.List[CoverageId]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class RequestValidationError(UniversalBaseModel):
    field_name: str = pydantic.Field(alias="fieldName")
    human_readable_message: typing.Optional[str] = pydantic.Field(alias="humanReadableMessage", default=None)

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .coverage_value import CoverageValue


class ServiceCoverageDetails(UniversalBaseModel):
    copay: typing.Optional[CoverageValue] = None
    coinsurance: typing.Optional[CoverageValue] = None
    visits: typing.Optional[CoverageValue] = None
    visits_remaining: typing.Optional[CoverageValue] = None
    additional_notes: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core.api_error import ApiError
from ..types.invalid_tag_names_error_type import InvalidTagNamesErrorType


class InvalidTagNamesError(ApiError):
    def __init__(self, body: InvalidTagNamesErrorType, headers: typing.Optional[typing.Dict[str, str]] = None):
        super().__init__(status_code=422, headers=headers, body=body)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .patient_payment_allocation import PatientPaymentAllocation


class PatientPaymentInfo(UniversalBaseModel):
    total_payment_cents: int = pydantic.Field()
    """
    The total amount of patient payments in cents for the service line.
    """

    items: typing.List[PatientPaymentAllocation] = pydantic.Field()
    """
    A list of patient payment items associated with the service line.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....commons.types.payer_plan_group_id import PayerPlanGroupId
from .....payers.resources.v_3.types.payer import Payer
from .mutable_payer_plan_group import MutablePayerPlanGroup


class PayerPlanGroup(MutablePayerPlanGroup):
    payer_plan_group_id: PayerPlanGroupId
    payer: Payer
    is_active: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
